//
//  JifChannel.cpp
//  glMovingLines
//
//  Created by Jiffer Harriman on 11/27/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#include "JifChannel.h"

//
//  JifChannelGui.cpp
//  glMovingLines
//
//  Created by Jiffer Harriman on 11/22/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//
// The JifChannelGui encapsulates the interaction and visualization for the a single channel of the Sinkapater

#include "JifChannelGui.h"

///////////////////////////////////////////////////////////////
// JifChannelGui master channel constructor
//
// id - a unique id which will be used as the start point for the OpenGL picking names
// offset - horizontal offset on the screen where this channel will reside
///////////////////////////////////////////////////////////////
JifChannel::JifChannel(int chanNum, float offset){
    float rate = init(chanNum, offset);
    m_chanSound = new MasterDrop(4, (int) rate, chanNum * 80); // create a harmonic series starting at 80 hz
    // this is the master channel so assign the pointer to one and the same dropper
    m_masterDrop = new MasterDrop;
    m_masterDrop = (MasterDrop *)m_chanSound;
    m_isMaster = true;
    setNumBeats(4);
    m_chanGui->setDropSpacing();
    
    // associate the sound dropper with each GUI element:
    setDropper();
}

///////////////////////////////////////////////////////////////
// JifChannelGui master channel constructor with audio file
//
// id - a unique id which will be used as the start point for the OpenGL picking names
// offset - horizontal offset on the screen where this channel 
JifChannel::JifChannel(int chanNum, float offset, AudioFile * theAudioFile){
    float rate = init(chanNum, offset);
    m_chanSound = new MasterDrop(4, (int) rate, theAudioFile);
    // this is the master channel so assign the pointer to one and the same dropper
    m_masterDrop = new MasterDrop;
    m_masterDrop = (MasterDrop *)m_chanSound;
    m_isMaster = true;
    setNumBeats(4);
    m_chanGui->setDropSpacing();
    setDropper();
}

JifChannel::JifChannel(MasterDrop * master, JifChannelGui * masterGui, int chanNum, float offset){
    float rate = init(chanNum, offset);
    m_chanSound = new SlaveDrop(master, INITDROPS, (int)rate, chanNum * 80);
    m_masterDrop = new MasterDrop;
    m_masterDrop = master;
    m_isMaster = false;
    setDropper();
    m_masterGui = new JifChannelGui(0, 0.0);
    m_masterGui = masterGui;
}

JifChannel::JifChannel(MasterDrop * master, JifChannelGui * masterGui, int chanNum, float offset, AudioFile * theAudioFile){
    float rate = init(chanNum, offset);
    m_chanSound = new SlaveDrop(master, INITDROPS, (int)rate, theAudioFile);
    m_masterDrop = new MasterDrop;
    m_masterDrop = master;
    m_isMaster = false;
    setDropper();
    m_masterGui = new JifChannelGui(0, 0.0);
    m_masterGui = masterGui;
}

float JifChannel::init(int chanNum, float offset){
    m_id = chanNum;
    m_chanGui = new JifChannelGui(chanNum, offset);

    return 0.1+(5.9* m_chanGui->getTopData()) * MY_SRATE; 

}

///////////////////////////////////////////////////////////////
// setWindowShape
// 
// called when the window size changes
// This scales the GUI chanel strip to the new format
///////////////////////////////////////////////////////////////

void JifChannel::setWindowShape(int w, int h){
    
    // update draw variables
    m_chanGui->setWindowShape(w, h);
    
    
}

///////////////////////////////////////////////////////////////
// display function
///////////////////////////////////////////////////////////////
void JifChannel::draw(GLenum mode){
    
    // draw the box around the channel
    m_chanGui->draw(mode, m_chanSound);
    
    if (!m_chanGui->getAnimate()){
    glPushMatrix();
    glTranslatef( m_chanGui->getHorizontal(),SINKBOTTOM-.1, 0);
    // print text:
    char printNum[32];
    sprintf(printNum, "%i", (int)m_chanSound->getNumBeats());
    strcat(printNum, " beats");
    printText(0, 0, 0, printNum);
    glPopMatrix();
    
    
    
    if (!m_isMaster){
        glPushMatrix();
        glTranslatef( m_chanGui->getHorizontal(),SINKBOTTOM-.2, 0);
        // print text:
        char printNumMaster[32];
        sprintf(printNumMaster, "%i", ((SlaveDrop *)m_chanSound)->getMasterNumBeats());
        strcat(printNumMaster, " master beats");
        printText(0, 0, 0, printNumMaster);
        glPopMatrix();
        
        glPushMatrix();
        glTranslatef( m_chanGui->getHorizontal(),SINKBOTTOM-.3, 0);
        // print text:
        char printBPMaster[32];
        sprintf(printBPMaster, "%f", ((float)m_masterDrop->getBeatRate() /  (float)((SlaveDrop *)m_chanSound)->getBeatRate()));
        strcat(printBPMaster, " beats per master beat");
        printText(0, 0, 0, printBPMaster);
        glPopMatrix();
        
        m_chanGui->drawMasterBeats();
    }
    else{
        glPushMatrix();
        glTranslatef( m_chanGui->getHorizontal(),SINKBOTTOM-.2, 0);
        // print text:
        char printBPM[32];
        sprintf(printBPM, "%f", 60 * MY_SRATE / ((float)m_masterDrop->getBeatRate()));
        strcat(printBPM, " beats per minute");
        printText(0, 0, 0, printBPM);
        glPopMatrix();
        

    }
    }    
}

///////////////////////////////////////////////////////////////
// motion function
///////////////////////////////////////////////////////////////
void JifChannel::procMovement(int x, int y){
    
    // process movement for the box
    bool dataChanged = m_chanGui->procMovement(x, y);
    
    //master
    if (m_isMaster)
    {    // calculate the new beatRate based on height of box
        float loopRate = 0.1+(5.9* m_chanGui->getTopData()) * MY_SRATE;
        if (loopRate < .1)
            loopRate = .1; // minimum loop rate
        m_chanSound->setLoopRate((int) loopRate);
    
        int numBeats = m_chanGui->getNumBeats();
        if (numBeats <= 0) numBeats = 1;
        else if (numBeats > MAXDROPS) numBeats =  MAXDROPS;
        m_chanSound->setNumBeats(numBeats);
        setDropper();
    }

    

   else{
       int loopRate =((SlaveDrop *)(m_chanSound))->getMasterNumBeats() * m_masterDrop->getBeatRate();
       // if master changed then the number of samples per beat has changed so need to update the slaves
       // set the new loop rate based on the master beat rate and num masterBeats
       if (((SlaveDrop *)(m_chanSound))->masterBeatRateChanged()){
           // new loop rate for each slave is
           // number of master beats * master beat rate
           loopRate = ((SlaveDrop *)(m_chanSound))->getMasterNumBeats() * m_masterDrop->getBeatRate();
       }
       
       // if slave changed
       // quantize the new loop rate to integer number of master beat rates
       else if (dataChanged)
       {
           // update top of box data (loop rate):
           loopRate = 0.1+(5.9* m_chanGui->getTopData()) * MY_SRATE;
           
           // calculate how many master beats this is closest to
           int numMasterBeats = (int) (loopRate / m_masterDrop->getBeatRate());
           
           // update the sound source master beats var1
           ((SlaveDrop *)(m_chanSound))->setMasterNumBeats(numMasterBeats);
           
           // quantize loop rate to this number of master beats
           loopRate = (int)(numMasterBeats * m_masterDrop->getBeatRate()); 
                                
       }
       // Update bottom of box data (number of beats)
       int numBeats = m_chanGui->getNumBeats();
       if (numBeats < 1) numBeats = 1;
       else if (numBeats > MAXDROPS) numBeats =  MAXDROPS;
       m_chanSound->setNumBeats(numBeats);
       
       // update the slave drop loop rate 
       if (loopRate < m_chanSound->getNumBeats()) loopRate = m_chanSound->getNumBeats(); // minimum loop rate = numBeats samples
       m_chanSound->setLoopRate(loopRate);
       
       //Quantize the height of the box
       float quantizedTop = ((loopRate - .1) * (SINKMAX-SINKMIN) + MY_SRATE * 5.9 * SINKMIN)/(5.9 * MY_SRATE);
       m_chanGui->setTop(quantizedTop);

   }
    // update position of the drops in the GUI:
    m_chanGui->setDropSpacing();
    
}

///////////////////////////////////////////////////////////////
// mouse function
///////////////////////////////////////////////////////////////
void JifChannel::pick(int button, int state, int x, int y){

    // box
    int pickedName = m_chanGui->pick(button, state, x, y);
    if (pickedName < MAXDROPS+1){
        if (pickedName > 3)
            m_chanSound->toggleBeat(pickedName - 4);
        else if (pickedName < 4){
            switch (pickedName) {
                case BOTTOM :
                    // Does this need to happen here?:
                    m_chanSound->setNumBeats(m_chanGui->getNumBeats());
                    break;
                    
                default:
                    break;
            }
        }
    }
}

///////////////////////////////////////////////////////////////
// procHits
// This takes in the OpenGL picking data and does something with it
///////////////////////////////////////////////////////////////
void JifChannel::procHits(GLint hits, GLuint buffer[])
{
    m_chanGui->procHits(hits, buffer);
}

///////////
void JifChannel::setNumBeats(int numBeats){
    
    if (numBeats > 0 && numBeats < MAXDROPS)
        m_numBeats = numBeats;
    
    m_chanSound->setNumBeats(numBeats);
    m_chanGui->setNumBeats(numBeats);
    m_chanGui->setBottomData(numBeats);
        m_chanGui->setDropSpacing();
}

//bool JifChannel::tick(SAMPLE * buffer, int bufferSize){
//    return m_chanSound->tick(buffer, bufferSize);
//}

void JifChannel::tick(SAMPLE * buffer, int bufferSize){
    if (m_chanSound->m_isMaster)
        m_chanSound->tick(buffer, bufferSize);
    else
        ((SlaveDrop *)m_chanSound)->tick(buffer, bufferSize);
}


void JifChannel::updateShit(){

    float br = m_masterDrop->getBeatRate() * ((SlaveDrop *)(m_chanSound))->getMasterNumBeats();
    //beatRate = m_numBeatsMaster * masterBeatRate; 
    m_chanSound->setLoopRate((int)br); // or what comes above?
    
    // quantize the height
    float quantizedTop = ((br - .1) * (SINKMAX-SINKMIN) + MY_SRATE * 5.9 * SINKMIN)/(5.9 * MY_SRATE);
    m_chanGui->setTop(quantizedTop);
    
    m_chanGui->setDropSpacing();
    
}
