//
//  JifChannel.h
//  glMovingLines
//
//  Created by Jiffer Harriman on 11/27/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

/// This class will encapsulate the GUI and the sequencers / sound making elements
/// in the functions which process the picking etc. it should call the GUI element and update the appropriate properties of the droppers
#ifndef JIFCHANNEL_H
#define JIFCHANNEL_H

#include "JifGL.h"
#include "JifChannelGui.h"
#include "JifDrop.h"
#include "SlaveDrop.h"
#include "MasterDrop.h"
#include "AudioFileSet.h"

class JifChannel {
    int m_numBeats;
    int m_id;
    float m_yInc;
    float m_chanWidth;
    bool * m_pickedArray;
    JifChannelGui * m_chanGui;
    JifDrop * m_chanSound;
    MasterDrop * m_masterDrop;
    JifChannelGui * m_masterGui;
    
    bool m_isMaster;
    
    
public:
    JifChannel(int chanNum, float position);
    JifChannel(int chanNum, float position, AudioFile * theAudioFile);
    JifChannel(MasterDrop * master, JifChannelGui * masterGui, int chanNum, float position);
    JifChannel(MasterDrop * master, JifChannelGui * masterGui, int chanNum, float position, AudioFile * theAudioFile);
    ~JifChannel(){};
    float init(int chanNum, float offset);
    void setDropper(){m_chanGui->setDropper(m_chanSound);};
    
    // reshape function
    void setWindowShape(int w, int h);
    
    // display function
    void draw(GLenum);
    
    // motion function
    void procMovement(int x, int y);
    
    // mouse function
    void pick(int button, int state, int x, int y);
    
    void procHits(GLint hits, GLuint buffer[]);
    
    void setNumBeats(int numBeats);
    
    // this is how many master beats the current chanel loops over:
    void setTopData(bool newMasterData);
    void updateShit();
    
    int getNumBeats(){return m_chanSound->getNumBeats();};
    int getBeatRate(){return m_chanSound->getBeatRate();};
    
    float getHorizontal(){return m_chanGui->getHorizontal();};
    
    MasterDrop * getMasterDrop(){return (MasterDrop *) m_chanSound;};
    JifChannelGui * getChannelGui(){return m_chanGui;};
    
    void tick(SAMPLE * buffer, int bufSize);
    
    void setAnimate(bool animate){m_chanGui->setAnimate(animate);};
    void setPlay(bool play){m_chanSound->setPlay(play);};
    
    void printText(int x, int y, int z, char *string)
    {
        glColor4f( 1, 1, 1, .5 );
        glRasterPos2f(x, y);
        int len, i;
        len = (int)strlen(string);
        for (i = 0; i < len; i++) {
            glutBitmapCharacter(GLUT_BITMAP_HELVETICA_10, string[i]);
        }
    }
};

#endif