//
//  JifChannelGui.cpp
//  glMovingLines
//
//  Created by Jiffer Harriman on 11/22/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//
// The JifChannelGui encapsulates the interaction and visualization for the a single channel of the Sinkapater

#include "JifChannelGui.h"

///////////////////////////////////////////////////////////////
// JifChannelGui constructor
//
// id - a unique id which will be used as the start point for the OpenGL picking names
// offset - horizontal offset on the screen where this channel will reside
///////////////////////////////////////////////////////////////

JifChannelGui::JifChannelGui(int id, float offset){
    m_numBeats = INITDROPS;// default number of beats in a new channel
    m_id = id;
    m_channelBox = new JifVerticalRect(m_id, offset);
    m_channelBox->setBottomData((float) INITDROPS);

    
    m_dropArray = new JifDropletGui * [MAXDROPS];
    
    // initialize the last picked array to all 0's
    m_pickedArray = new bool[MAXDROPS];
    clearLastPicked();
    
    // calc droplet locations
    m_chanWidth = m_channelBox->getRectWidth();
    m_yInc =  (m_channelBox->getRectHeight()) / (m_numBeats+1);
    
    for (int i = 0; i < MAXDROPS; i++){
        m_dropArray[i] = new JifDropletGui(m_id, i, offset + m_chanWidth/2, (i+1) * m_yInc + SINKBOTTOM);
    }
    
    m_animate = 0;
}

///////////////////////////////////////////////////////////////
// setWindowShape
// 
// called when the window size changes
// This scales the GUI chanel strip to the new format
///////////////////////////////////////////////////////////////

void JifChannelGui::setWindowShape(int w, int h){
    
    // update draw variables
    m_channelBox->setWindowShape(w, h);
    
    for (int i = 0; i < m_numBeats; i++){
        m_dropArray[i]->setWindowShape(w, h);
    }
    
}

///////////////////////////////////////////////////////////////
// display function
///////////////////////////////////////////////////////////////
void JifChannelGui::draw(GLenum mode, JifDrop * thisDrop){
    if (m_animate)
    {
        m_channelBox->animate(m_yInc * (m_numBeats) + SINKBOTTOM );

        // HACK!
        int currentBeat; 
        if(thisDrop->m_useSample)
            currentBeat = thisDrop->getCurrentBeat()+1;
        else
            currentBeat = thisDrop->getCurrentBeat();
        float beatOffset = ( m_yInc * thisDrop->getSampleCount()) / thisDrop->getBeatRate();
        for (int i = 0; i < m_numBeats; i++){
            m_dropArray[(i+currentBeat) % m_numBeats]->setYOffset((i+1) * m_yInc + SINKBOTTOM - beatOffset);
            m_dropArray[i]->animate();
        }
    }
    
    else{
        // draw the box around the channel
        m_channelBox->draw(mode);
        // draw each droplet
        for (int i = 0; i < m_numBeats; i++){
            m_dropArray[i]->draw(mode);
        }
        
    }
   
}

// draw lines to indicate where the msater beats are relative to this channels beats
void JifChannelGui::drawMasterBeats(){
    if(!m_animate){
        
        //glColor3b(, 0.25, 0.25);
        float beatH_inc = ((SlaveDrop *)m_theDropper)->getBeatsPerMaster();
        float beatH = (SINKBOTTOM + m_yInc);
        
        // don't draw the lines if they get too close together
        if (beatH_inc > .2){
        while (beatH<m_channelBox->getTop()){
            
        
            glBegin(GL_LINES);
            glVertex3f((m_channelBox->getHorizontal()), beatH, 0);
            glVertex3f(((m_channelBox->getHorizontal()) + .05 ), beatH, 0);
            glEnd();
    
            
            beatH += m_yInc * beatH_inc;
        }
        }
    }
}

///////////////////////////////////////////////////////////////
// motion function
///////////////////////////////////////////////////////////////
bool JifChannelGui::procMovement(int x, int y){
    
    // process movement for the box
    bool returnVal = m_channelBox->procMovement(x, y);
    
    // update number of beats
    setNumBeats((int)m_channelBox->getBottomData());

    return returnVal;
}

void JifChannelGui::setDropSpacing(){
    // in case this box is changing size find the new increment
    m_yInc =  (float)(m_channelBox->getRectHeight()) / (m_numBeats+1);
    
    
    // update for each droplet
    for (int i = 0; i< m_numBeats; i++){
        m_dropArray[i]->setYOffset((i+1) * m_yInc + SINKBOTTOM);
        // m_dropArray[i]->procMovement(x, y);
    }
}

///////////////////////////////////////////////////////////////
// mouse function
// returns the index of the beat which has been toggled when clicked
///////////////////////////////////////////////////////////////
int JifChannelGui::pick(int button, int state, int x, int y){
    
    // box
    int boxPick = m_channelBox->pick(button, state, x, y);
    
    if (boxPick < 4)
        return boxPick;//////////////////

    // else check droplets :   
    clearLastPicked();
    
    GLuint selectBuf[JIFPICKBUFSIZE];
    GLint hits;
    GLint viewport[4];
    
    if (button != GLUT_LEFT_BUTTON || state != GLUT_DOWN)
        return MAXDROPS+5;
    
    glGetIntegerv(GL_VIEWPORT, viewport);
    
    glSelectBuffer(JIFPICKBUFSIZE, selectBuf);
    
    // Change to Select mode
    (void) glRenderMode(GL_SELECT);
    
    glInitNames();
    glPushName(0);
    
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    
    /*  create 10x10 pixel picking region near cursor location */
    gluPickMatrix((GLdouble) x, (GLdouble) (viewport[3] - y), 10.0, 10.0, viewport);
    glOrtho(0.0, 8.0, 0.0, 8.0, -0.5, 2.5);
    
    // call draw function in select mode:
    JifDrop * dummy;
    draw(GL_SELECT, dummy);
    glPopMatrix();
    glFlush();
    
    hits = glRenderMode(GL_RENDER);
    
    // process the hits returned
    // TODO make conditional so it only queries when necessary
    
    if (selectBuf[3] > m_id * NAMEINCREMENT + 3)
        procHits(hits, selectBuf);
    
    if (hits > 0)
        return (int)((selectBuf[3] % (m_id * NAMEINCREMENT))); // need to offset on account of picking names
    else return MAXDROPS+5;
    
}

///////////////////////////////////////////////////////////////
// clearLastPicked()
// This is called to clear out the picking buffer so each time a new object is picked it is unique
///////////////////////////////////////////////////////////////
void JifChannelGui::clearLastPicked()
{
    for (int i = 0; i< m_numBeats; i++){
        m_pickedArray[i] = false; }
};

///////////////////////////////////////////////////////////////
// procHits
// This takes in the OpenGL picking data and does something with it
///////////////////////////////////////////////////////////////
void JifChannelGui::procHits(GLint hits, GLuint buffer[])
{
    unsigned int i, j;
    GLuint names, *ptr;
    
//  printf("hits = %d\n", hits);
    ptr = (GLuint *) buffer;
    for (i = 0; i < hits; i++) {  /* for each hit  */
        names = *ptr;
//        printf(" number of names for hit = %d\n", names); ptr++;
//        printf("  z1 is %g;", (float) *ptr/0x7fffffff); ptr++;
//        printf(" z2 is %g\n", (float) *ptr/0x7fffffff); ptr++;
//        printf("   the name is ");
//        for (j = 0; j < names; j++) {  /* for each name */
//            printf("%d ", *ptr); ptr++;
//        }
//        printf("\n");
    }
    
    // check if there was a new hit 
    // if so set the appropriate pickArray buffer to true
    if (hits > 0){
        m_pickedArray[(buffer[3] % (m_id * NAMEINCREMENT))] = 1;
        // subtract 4 to compensate for naming offset when toggling state
        m_dropArray[(buffer[3] % (m_id * NAMEINCREMENT)) - 4]->setEnabled();
    }
}

///////////
void JifChannelGui::setNumBeats(int numBeats){

    if (numBeats > 0 && numBeats < MAXDROPS)
        m_numBeats = numBeats;
}

void JifChannelGui::setTop(float h){
    m_channelBox->setTop(h);
}

void JifChannelGui::setAnimate(bool animate){
    m_animate = animate;
    if(!m_animate){
        for (int i = 0; i < m_numBeats; i++){
            m_dropArray[(i) % m_numBeats]->setYOffset((i+1) * m_yInc + SINKBOTTOM);
        }
    }
}

