//
//  JifDrop.cpp
//  
//
//  Created by Jiffer Harriman on 11/9/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//
// The JifDrop class is a beat dropper.  It keeps time and makes noise at a given interval.  
// There is method to change this interval.  There is a method to update which beats to sound on.  
// There is a method to change the velocity of a given beat.
//
#include "JifDrop.h"


JifDrop::JifDrop()
{
    int numBeats = INITDROPS; 
    int loopRate = MY_SRATE;
    init(numBeats, loopRate, 200);
}

JifDrop::JifDrop(int numBeats, int loopRate)
{
    init(numBeats, loopRate, 200);
}

JifDrop::JifDrop(int numBeats, int loopRate, float freq)
{
    init(numBeats, loopRate, freq);
}

JifDrop::JifDrop(int numBeats, int loopRate, AudioFile * theFile){
    m_audio = theFile;
    init(numBeats, loopRate, 0);
    
    m_useSample = 1;
}


// init()
// This function initializes the member variables
// pass it the number of beats to use over the loopRate.  The loopRate (in samples) 
// and a frequency for the sound synthesis
void JifDrop::init(int numBeats, int loopRate, float freq)
{
    m_sampleCount = 0;
    m_gain = 0; 
    m_currentBeat = 0;

    // initialize ADSR envelope
    m_adsr = new ADSR();
    m_adsr->setAllTimes(0.001, 0.02, 0.0, 0.1);
    m_adsr->keyOff();
    
    // init sounds
    m_blit = new Blit();
    m_blit->setFrequency(freq);
    m_blit->setHarmonics(10);
    setFreq(freq);    
    
    // Reverb:
    m_rev = new JCRev();
    m_rev->setT60(3.0); // perhaps do based on zoom level?
    m_rev->setEffectMix(0.2);

    m_pattern = new float[MAXDROPS];
    m_beatEnabled = new bool[MAXDROPS];
    //initialize pattern with a sound on the first beat only
    //m_pattern[0] = .5;
    for(int i = 0; i<MAXDROPS; i++)
    {
        setBeatValue(i, 0);
        m_beatEnabled[i] = false;
    }
    
    m_play = 1;
    m_mute = 0;
    
    setNumBeats(numBeats);
    setLoopRate(loopRate);
    
    m_useSample = 0;
    m_isMaster = 0;
}



// setBeatValue()
// use this method to update the velocity at a given beat increment
void JifDrop::setBeatValue(int index, float newValue)
{
    // error checking
    if (index > m_numBeats){
        //cerr << "Warning: setBeatValue(index, value) - Beat index must be less than number of beats (starting at 0)\n";
        return;
    }
    // normalize value to 0 to 1 range for gain
    // TODO: if using midi data use as attack value
    if (newValue > 0)
    {
        if (newValue > 127)
            newValue = 127;
        m_pattern[index] = newValue;
    }
    
    
}

// setNumBeats()
// update the number of beats used to divide up the loopRate interval by
void JifDrop::setNumBeats(int numBeats)
{   // calc old beat rate:
    int oldRepeatRate = getLoopRate();
    
    // temporarily store old number of beats and update m_numBeats
    //int oldNumBeats = m_numBeats;
    m_numBeats = numBeats;
    
    // set new beat rate:
    m_beatRate = oldRepeatRate / m_numBeats;
    
}

// Go to the next sample step, beat interval and trigger sound if necessary
void JifDrop::advance()
{  
    // Check if the sample count is equal to the beat rate in samples and if the current beat is enabled    
    if (m_sampleCount >= m_beatRate)
    {
        // reset sample count
        m_sampleCount = 0;
        
        // if beat is enabled play it:
        if (m_beatEnabled[m_currentBeat] == true)
            m_adsr->keyOn();
        else
            m_adsr->keyOff();
        
        // increment the current beat
        m_currentBeat++;
        if (m_currentBeat >= m_numBeats)
            m_currentBeat = 0;
    }
    m_sampleCount++;
}
//
//
//
//bool JifDrop::tick(SAMPLE * outputBuffer, unsigned int bufferSize)
//{
//    if (m_play){
//        for (int i = 0; i<bufferSize; i++){
//            advance(0);
//            
//            if (m_useSample){
//                if(m_sampleCount < m_audio->lengthSamps/m_audio->channels)
//                    outputBuffer[i] = m_rev->tick(m_beatEnabled[m_currentBeat] * m_audio->wave[m_audio->channels * m_sampleCount]);
//                else
//                    outputBuffer[i] = m_rev->tick(0);
//            }
//            else
//                outputBuffer[i] = m_rev->tick( m_adsr->tick() * ( m_blit->tick()));     
//        }
//    }
//    return m_Trigger;
//}

void JifDrop::tick(SAMPLE * outputBuffer, unsigned int bufferSize)
{
    if (m_play){
        for (int i = 0; i<bufferSize; i++){
            //outputBuffer[i] = m_pattern[m_currentBeat] * m_adsr->tick() * ( m_blit->tick());    
            advance();
            if (m_useSample){
                if(m_sampleCount < m_audio->lengthSamps/m_audio->channels)
                    outputBuffer[i] = m_rev->tick(m_beatEnabled[m_currentBeat] * m_audio->wave[m_audio->channels * m_sampleCount]);
                else
                    outputBuffer[i] = m_rev->tick(0);
            }
            else
                outputBuffer[i] = m_rev->tick( m_adsr->tick() * ( m_blit->tick())); 
        }
        
    }
    
    if (m_mute)
    {
        for(int i = 0; i<bufferSize; i++)
            outputBuffer[i] = 0;
    }
    
    }

void JifDrop::setPlay(bool play){
    m_play = play;
    if (!m_play){
        m_sampleCount = 0;
        m_gain = 0; 
        m_currentBeat = 0;
        //m_currentBeatMaster = 0;
        //m_Trigger = 0;
    }
}