//
//  JifDrop.h
//  
//
//  Created by Jiffer Harriman on 11/9/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//
// The JifDrop class is a beat dropper.  It keeps time and makes noise at a given interval.  
// There is method to change this interval.  There is a method to update which beats to sound on.  
// There is a method to change the velocity of a given beat.


#ifndef __DROP_H__
#define __DROP_H__

#include <iostream>
#include <math.h>
#include "setup.h"
#include "JifGL.h"
#include "AudioFileSet.h"

#include "stk/ADSR.h"
#include "stk/Blit.h"
#include "stk/JCRev.h"

using namespace std;
using namespace stk;


// Signal Generator parent class
class JifDrop
{
public:
    JifDrop();
    JifDrop(int numBeats, int repeatRate);
    JifDrop(int numBeats, int repeatRate, float freq);
    JifDrop(int numBeats, int repeatRate, AudioFile * audioFile);
    ~JifDrop(){};
    void init(int numBeats, int repeatRate, float freq);
    void trigger(float gain);
    void adsr();
    void tick(SAMPLE * buffer, unsigned int size);
    //void tick(SAMPLE * buffer, unsigned int size, bool masterTrigger); instead slave can just look at the master's sampleCount var
    void drip();
    //void advance(bool masterTrigger);
    void advance();
    
    // Set functions
    void setLoopRate(int loopRate){m_beatRate = loopRate / m_numBeats;};
    void setNumBeats(int numBeats);
    void setBeatValue(int index, float newValue);
    void resetSampleCount(){m_sampleCount = 0;};
    void setToggleValue(int index, bool enabled){m_beatEnabled[index] = enabled;};
    void toggleBeat(int index){m_beatEnabled[index] = !m_beatEnabled[index];};
    void toggleMute(){m_mute = !m_mute;};
    void setFreq(float freq){m_blit->setFrequency(freq);};
    void setPlay(bool play);
    
    //Get Functions
    int getNumBeats(){return m_numBeats;};
    int getBeatRate(){return m_beatRate;};
    float * getPattern(){return m_pattern;};
    int getLoopRate(){return m_beatRate * m_numBeats;};
    int getCurrentBeat(){return m_currentBeat;};
    int getSampleCount(){return m_sampleCount;};
    bool getMute(){return m_mute;};
    
    bool m_isMaster;
    bool m_useSample; // otherwise use blit for sound source
    // member variables
protected:
    int m_beatRate;
    int m_numBeats;
//    int m_numBeatsMaster; move to slave
    
//    int m_currentBeatMaster;
    int m_sampleCount;
//    bool m_Trigger;  // there must be a better way
    
    float * m_pattern;        
    bool * m_beatEnabled;
    float m_gain;
    int m_currentBeat; // pointer to pattern array
    
    bool m_play;
    bool m_mute;

    ADSR * m_adsr;
    Blit * m_blit;
    JCRev * m_rev;
    
    AudioFile * m_audio;
};

#endif