//
//  JifKitchen.cpp
//  glMovingLines
//
//  Created by Jiffer Harriman on 12/8/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#include "JifKitchen.h"

JifKitchen::JifKitchen(int bufSize){
    
    theSinks = new Sinkapater * [NUMSINKS];
    theFileSets = new AudioFileSet * [NUMSINKS];
    
    for (int i = 0; i<NUMSINKS; i++){
        theFileSets[i] = new AudioFileSet();
    }
    
    theFileSets[0]->loadFileSet("../../samples/water/");
    theFileSets[1]->loadFileSet("../../samples/drums/");
    theFileSets[2]->loadFileSet("../../samples/piano/");
    
    for (int i = 0; i< NUMSINKS; i++){
        int NChan = theFileSets[i]->getNumFiles();
        vector<AudioFile *> * audioFilesPtr;
        audioFilesPtr = theFileSets[i]->getFileVector();
        
        // for the first sinke generate the master channel:
        if (i == 0){
            if (NChan>0)
                theSinks[i] = new Sinkapater(NChan, bufSize, audioFilesPtr);
            else
                theSinks[i] = new Sinkapater(INITCHANS, bufSize);
        }
        else {
            if (NChan>0)
                theSinks[i] = new Sinkapater(theSinks[0]->getMasterDrop(), theSinks[0]->getFirstChannel(), NChan, bufSize, audioFilesPtr);
            else
                theSinks[i] = new Sinkapater(theSinks[0]->getMasterDrop(), theSinks[0]->getFirstChannel(), INITCHANS, bufSize);
        }
    }
    
    m_mixBufArray = new SAMPLE * [NUMSINKS];
    
    for (int i = 0; i < NUMSINKS; i++){
        m_mixBufArray[i] = new SAMPLE[NCHAN * bufSize];
    }
    
    // initialize animated drop color:
    m_color = new GLfloat[4];
    m_color[0] = 0.0;
    m_color[1] = 0.2;
    m_color[2] = 0.8;
    m_color[3] = 0.5;
    
    m_viewDir = 0;
}

void JifKitchen::draw(int current){
    if (theSinks[0]->getAnimate()){
        glPushMatrix();
        
        switch (m_viewDir) {
            case 0:
                glTranslated(0, -1, -5);
                glRotated(0, 0, 1, 0);
                
                break;
                
            case 1:
                glTranslated(15, -1, -25);
                glRotated(90, 0, 1, 0);
                break;
            case 2:
                glTranslated(8, -1, -35);
                glRotated(180, 0, 1, 0);
                break;
            case 3:
                glTranslated(-15, -1, -25);
                glRotated(-90, 0, 1, 0);
                break;
                
            default:
                break;
        }
        
         
        glMaterialfv(GL_FRONT,GL_DIFFUSE,m_color);
        // draw the bottom surface
        glBegin(GL_QUADS);
        glColor3b(1.0, 0, 0);
        glVertex3f(-2.0, 0.0, -2.0);
        glVertex3f(-2.0, 0.0, -37.0);
        glVertex3f(SCREENNORM+2, 0.0, -37.0);
        glVertex3f(SCREENNORM+2, 0.0, -2.0);
        glEnd();

        float offset = -5.0;
        float zinc = -10.0;

        for (int i = 0; i < NUMSINKS; i++){
            glPushMatrix();
            glTranslatef(0.0, 0.0, offset);

            theSinks[i]->draw(GL_RENDER);
            glPopMatrix();
            offset += zinc;
        }
        glPopMatrix();
    }
    else
        theSinks[current]->draw(GL_RENDER);
}


void JifKitchen::toggleAnimate(){
    for (int i = 0; i < NUMSINKS; i++){
        theSinks[i]->toggleAnimate();
    }
}

void JifKitchen::togglePlay(){
    for (int i = 0; i < NUMSINKS; i++){
        theSinks[i]->togglePlay();
    }
}

void JifKitchen::tick(double *buffer, int bufferSize){
    // clear out the buffer
    for (int i = 0; i<bufferSize; i++){
        buffer[i*NCHAN] = 0;
        for (int j = 1; j<NCHAN; j++)
            buffer[NCHAN*i+j] = buffer[NCHAN*i];
    }
    
    // Tick the sinks and sum from the mix buffer
    for (int j = 0; j < NUMSINKS; j++){
        theSinks[j]->tick(m_mixBufArray[j], bufferSize);
        for (int k = 0; k<bufferSize; k++){
            buffer[k*NCHAN] += m_mixBufArray[j][k*NCHAN];

            for (int j = 1; j<NCHAN; j++)
                buffer[NCHAN*k+j] = buffer[NCHAN*k];
        }
    }
}