//
//  JifPickLine.cpp
//  glMovingLines
//
//  Created by Jiffer Harriman on 11/18/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//
//  JifRect Class
//  This OpenGL object can be reshaped by clicking and dragging one of the four lines that make it up

#include "JifRect.h"

JifRect::JifRect()
{
    m_id = 1;
    init(WIDTH, HEIGHT, 0.2, 0.0, 0.1, SINKTOP, SINKBOTTOM);
}

JifRect::JifRect(int rect_id, float offset){
    m_id = rect_id;
    init(WIDTH, HEIGHT, offset, 0.0, 0.2, SINKTOP, SINKBOTTOM);
}

JifRect::JifRect(int rect_id, float offset, float left, float right, float top, float bottom){
    m_id = rect_id;
    init(WIDTH, HEIGHT, offset, left, right, top, bottom);
}

JifRect::~JifRect()
{
    
}

void JifRect::init(int screenWidth, int screenHeight, float offset, float left, float right, float top, float bottom)
{
    // set rectangle color (light grey)
    m_color_r = 0.25;
    m_color_g = 0.25;
    m_color_b = 0.25;
    
    // initial position
    m_left = (float) offset + left;
    m_right = (float) offset + right;
    m_top = (float) top;
    m_bottom = (float) bottom;
    
    m_horizontalOffset = offset;
    
    // initialize the last picked array to all 0's
    m_pickedArray = new bool[4];
    clearLastPicked();
    
    m_screenWidth = screenWidth;
    m_screenHeight = screenHeight;
    
    // calculate the scale factor
    // this is used to determine how much to move the edges by based on the window size
    m_screenScaleW = SCREENNORM / m_screenWidth;
    m_screenScaleH = SCREENNORM / m_screenHeight;
    m_theDrop = new JifDrop();
}

void JifRect::pick(int button, int state, int x, int y)
{
    clearLastPicked();
    
    
    if (state == GLUT_DOWN)
        m_buttonState = 1;
    else
        m_buttonState = 0;
    
    GLuint selectBuf[JIFPICKBUFSIZE];
    GLint hits;
    GLint viewport[4];
    
    if (button != GLUT_LEFT_BUTTON || state != GLUT_DOWN)
        return;
    
    glGetIntegerv(GL_VIEWPORT, viewport);
    
    glSelectBuffer(JIFPICKBUFSIZE, selectBuf);

    // Change to Select mode
    (void) glRenderMode(GL_SELECT);
    
    glInitNames();
    glPushName(0);
    
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    
    /*  create 10x10 pixel picking region near cursor location */
    gluPickMatrix((GLdouble) x, (GLdouble) (viewport[3] - y), 10.0, 10.0, viewport);
    glOrtho(0.0, 8.0, 0.0, 8.0, -0.5, 2.5);
    
    // call draw function in select mode:
    draw(GL_SELECT);
    glPopMatrix();
    glFlush();
    
    hits = glRenderMode(GL_RENDER);
    
    // process the hits returned
    procHits(hits, selectBuf);
    
    
}

void JifRect::draw(GLenum mode)
{
    glColor3f(m_color_r, m_color_g, m_color_b);
    
    if (mode == GL_SELECT)
        glLoadName(m_id * NAMEINCREMENT + TOP); // increment by NAMEINCREMENT so names don't clobber each other
    glBegin(GL_LINES);
    glVertex3f((m_left), m_top, 0);
    glVertex3f((m_right ), m_top, 0);
    glEnd();
    
    if (mode == GL_SELECT)
        glLoadName(m_id * NAMEINCREMENT + RIGHT);
    glBegin(GL_LINES);
    glVertex3f((m_right ), m_top, 0);
    glVertex3f((m_right), m_bottom, 0);
    glEnd();
    
    if (mode == GL_SELECT)
        glLoadName(m_id * NAMEINCREMENT + BOTTOM);
    glBegin(GL_LINES);
    glVertex3f((m_right), m_bottom, 0);
    glVertex3f((m_left ), m_bottom, 0);
    glEnd();
    

    if (m_theDrop->getMute())
        glColor3f(1.0, 0.0, 0.0);
    if (mode == GL_SELECT)
        glLoadName(m_id * NAMEINCREMENT + LEFT);
    glBegin(GL_LINES);
    glVertex3f((m_left ), m_bottom, 0);
    glVertex3f((m_left ), m_top, 0);
    glEnd();
}

void JifRect::procHits(GLint hits, GLuint buffer[])
{
    unsigned int i, j;
    GLuint names, *ptr;
        
    //printf("hits = %d\n", hits);
    ptr = (GLuint *) buffer;
    for (i = 0; i < hits; i++) {  /* for each hit  */
        names = *ptr;
//        printf(" number of names for hit = %d\n", names); ptr++;
//        printf("  z1 is %g;", (float) *ptr/0x7fffffff); ptr++;
//        printf(" z2 is %g\n", (float) *ptr/0x7fffffff); ptr++;
//        printf("   the name is ");
//        for (j = 0; j < names; j++) {  /* for each name */
//            printf("%d ", *ptr); ptr++;
//        }
//        printf("\n");
    }
    
    // check if there was a new hit 
    // if so set the appropriate buffer to true
    if (hits > 0 && (buffer[3] % (m_id * NAMEINCREMENT)) < 4){
        m_pickedArray[buffer[3] % (m_id * NAMEINCREMENT)] = 1;
        if (buffer[3] % (m_id * NAMEINCREMENT) == LEFT)
            m_theDrop->toggleMute();
    }
}

void JifRect::procMovement(int x, int y)
{
    for (int i = 0; i<4; i++){
        if (m_pickedArray[i] == 1)
            switch (i) {
                case TOP:
                    m_top = (m_screenHeight - y) * m_screenScaleH;
                    break;
                case BOTTOM:
                    m_bottom = (m_screenHeight - y) * m_screenScaleH;
                    break;
                case RIGHT:
                    m_right = x * m_screenScaleW;
                    break;
                case LEFT:
                    m_left = x * m_screenScaleW;
                    break;
                    
                default:
                    break;
            }
    }
     
}
    
void JifRect::clearLastPicked()
{
    for (int i = 0; i<4; i++){
    m_pickedArray[i] = 0; }
};

/// update the window size variables
void JifRect::setWindowShape(int w, int h)
{
    m_screenHeight = h;
    m_screenWidth = w;
    m_screenScaleW = SCREENNORM / w;
    m_screenScaleH = SCREENNORM / h;
};
