//
//  Sinkapater.cpp
//  glMovingLines
//
//  Created by Jiffer Harriman on 11/18/11.
//  Copyright 2011 __MyCompanyName__. All rights reserved.
//

#include "Sinkapater.h"

// Constructor without audio file pointer or master drop handle
// if it isn't passed a pointer to a master dropper then it should generate the master
Sinkapater::Sinkapater(int NChan, int bufSize){
    init(NChan, bufSize);
    
    float xinc = SCREENNORM/(m_numFaucets+1);
    
    // create master Channel:
    m_chanArray[0] = new JifChannel(1, xinc);

    // assign master drop pointer
    m_masterDrop = m_chanArray[0]->getMasterDrop();
    
    for (int i = 1; i < NChan-1; i++){
        m_chanArray[i] = new JifChannel(m_masterDrop, m_chanArray[0]->getChannelGui(), (i+1), (i+1) * xinc);
    }    
}

Sinkapater::Sinkapater(int NChan, int bufSize, vector<AudioFile *> * audioFiles){
    init(NChan, bufSize);
    
    float xinc = SCREENNORM/(m_numFaucets+1);
    // create master Channel:
    m_chanArray[0] = new JifChannel(1, xinc, audioFiles->at(0));
    
    // assign master drop pointer
    m_masterDrop = m_chanArray[0]->getMasterDrop();
    
    for (int i = 1; i < NChan; i++){
        m_chanArray[i] = new JifChannel(m_masterDrop, m_chanArray[0]->getChannelGui(), (i+1), (i+1) * xinc, audioFiles->at(i));
    } 
}

Sinkapater::Sinkapater(JifDrop * masterDrop, JifChannel * masterChannel, int NChan, int bufSize){
    init(NChan, bufSize);

    // assign master drop pointer
    m_masterDrop = (MasterDrop *)masterDrop;
    m_masterChannel = masterChannel;
    
    float xinc = SCREENNORM/(m_numFaucets+1);
    
    for (int i = 0; i < NChan; i++){
        m_chanArray[i] = new JifChannel(m_masterDrop, m_masterChannel->getChannelGui(), (i+1), (i+1) * xinc);
    }    
}

Sinkapater::Sinkapater(JifDrop * masterDrop, JifChannel * masterChannel, int NChan, int bufSize, vector<AudioFile *> * audioFiles){
    init(NChan, bufSize);
    
    // assign master drop pointer
    m_masterDrop = (MasterDrop *)masterDrop;
    m_masterChannel = masterChannel;
    
    float xinc = SCREENNORM/(m_numFaucets+1);
        
    for (int i = 0; i < NChan; i++){
        m_chanArray[i] = new JifChannel(m_masterDrop, m_masterChannel->getChannelGui(), (i+1), (i+1) * xinc, audioFiles->at(i));
    }
    
}


void Sinkapater::init(int NChan, int bufSize){
    // one less since the masterbuffer is not part of the array that this is used to address
    m_numFaucets = NChan; 
    m_chanArray = new JifChannel * [NChan];
    m_mixBufArray = new SAMPLE * [m_numFaucets];
    
    m_bufferSize = bufSize;
    
    for (int i = 0; i < NChan; i++){
        m_mixBufArray[i] = new SAMPLE[m_bufferSize];
    }
    
    // set up windows:
    m_window = new float[m_bufferSize];
    m_taperWindow = new float[NUMRINGS*2];
    hamming(m_window, (unsigned long) m_bufferSize);
    hamming(m_taperWindow, (unsigned long) NUMRINGS*2);
    
    m_rmsBuffer = new SAMPLE * [m_numFaucets];
    for (int i = 0; i< m_numFaucets; i++){
        m_rmsBuffer[i] = new SAMPLE[NUMRINGS];

    }
    m_rmsPointer = 0;
    
    // initialize animate variable
    // start in edit mode
    m_animate = 0;
    // start out playing
    m_play = 1;

    // initialize animated drop color:
    m_color = new GLfloat[4];
    m_color[0] = 0.2;
    m_color[1] = 0.2;
    m_color[2] = 1.0;
    m_color[3] = 0.5;
}

void Sinkapater::setWindowShape(int w, int h)
{
    for (int i = 0; i< m_numFaucets; i++){
        m_chanArray[i]->setWindowShape(w,h);
    }
}

void Sinkapater::draw(GLenum mode){
    
    if( m_animate){
        glPushMatrix();

        for (int i = 0; i < m_numFaucets; i++){
            m_chanArray[i]->draw(mode);
        }
        //drawWaves();
        //drawRings();
        drawRings();
        
        glPopMatrix();
    }
    else
    {
        for (int i = 0; i < m_numFaucets; i++){
            m_chanArray[i]->draw(mode);
        }
    }

}


// drawWaves()
// This function goes through the mixer audio buffer array and draws whatever is currently in there
void Sinkapater::drawWaves(){
    // There must be a better way
    // I'm disabling these modes here because my lines don't get rendered otherwise:
    glDisable(GL_LIGHTING);
    glDisable(GL_LIGHT0);
    glDisable(GL_DEPTH_TEST);
    
    // TODO: automate these start positions:
    float xinc = ((SCREENNORM - 1.0) / (INITCHANS+1))/m_bufferSize;
    float xpos = .1-(xinc * m_bufferSize)/2; // plus half the width of the channel to put it in the middle
    
    glLineWidth(1.0);
    glColor4f(0.2f, 0.2f, 0.8f, 0.1f); 
    
    for (int j = 0; j<m_numFaucets; j++){
        // reset the x position:
        xpos = .1-(xinc * m_bufferSize)/2; // plus half the width of the channel to put it in the middle
        
        // draw the wave form for the rest of the channels:
        glPushMatrix();
        glTranslatef(m_chanArray[j]->getHorizontal(), SINKBOTTOM, 0);
        glBegin(GL_LINES); 
        for(int i = 0; i < m_bufferSize; i++) { 
            
            glVertex3f(xpos, 0.5 * m_window[i] * m_mixBufArray[j][i], 0); 
            xpos = xinc + xpos;
        } 
        glEnd();
        glPopMatrix();

    }

    glEnable(GL_LIGHTING);
    glEnable(GL_LIGHT0);
    glEnable(GL_DEPTH_TEST);
}

void Sinkapater::drawRings(){
    float radius;
    float rad_inc = 1.0 / (NUMRINGS);
    
    glLineWidth(1.0);
    glColor4f(0.2f, 0.2f, 0.8f, 0.1f); 
    glMaterialfv(GL_FRONT,GL_DIFFUSE,m_color);
    
    for (int j = 0; j<m_numFaucets; j++){
        radius = 0.0;
        
        m_rmsBuffer[j][m_rmsPointer] = 0;
        // calculate RMS value and store it in the latest buffer
        for(int i = 0; i < m_bufferSize; i++){
            // sum the samples
            m_rmsBuffer[j][m_rmsPointer] += pow(m_mixBufArray[j][i], 2);
        }
        m_rmsBuffer[j][m_rmsPointer] = sqrt(m_rmsBuffer[j][m_rmsPointer] / m_bufferSize);
        
        glPushMatrix();
        glTranslatef(0.1 + m_chanArray[j]->getHorizontal(), SINKBOTTOM, 0);
        
        
        
        for(int i = 0; i < NUMRINGS; i++) { 
            int buf_index = NUMRINGS + m_rmsPointer - i;
            buf_index %= NUMRINGS;
            
//            glBegin(GL_LINE_LOOP);
//            for(int k = 0; k < 100; k++) { 
//                glVertex3f((cos(angle) * (radius)), 0.5 * m_rmsBuffer[j][buf_index],  (sin(angle) * (radius))); 
//                angle += angle_inc; 
//                // vertex buffer object for speed
//                //gluCylinder()
//            }           
//            glEnd(); 
//            radius +=rad_inc;
            
            glPushMatrix();
            glRotated(90, 1.0, 0.0, 0.0);
            glTranslatef(0.0, -4.0 * m_rmsBuffer[j][buf_index] * m_taperWindow[i+NUMRINGS], 0.0);
            glutWireTorus(rad_inc, radius, 5, 19);
            glPopMatrix();
            
            radius +=rad_inc;
        }
        glPopMatrix();
    }
    m_rmsPointer++;
    m_rmsPointer %= NUMRINGS;
}

void Sinkapater::pick(int button, int state, int x, int y){
    // ignore if in animate mode
    if (!m_animate){
        for (int i = 0; i < m_numFaucets; i++){
            m_chanArray[i]->pick(button, state, x, y);
        }
    }
}

void Sinkapater::procMovement(int x, int y){
    // ignore if in animate mode
    if (!m_animate){
        for (int i = 0; i < m_numFaucets; i++){
            m_chanArray[i]->procMovement(x, y);
        }
    }
}

void Sinkapater::tick(SAMPLE * buffer, int bufferSize){
        
    // clear out the buffer
    for (int i = 0; i<bufferSize; i++){
        buffer[i*NCHAN] = 0;
        for (int l = 1; l<NCHAN; l++)
            buffer[NCHAN*i+l] = buffer[NCHAN*i];
    }
    
    if (m_play){
    //TODO: Should add some panning based on where it is    
    for (int j = 0; j<(m_numFaucets); j++){
//  for panning:
//        float offset;
//        offset = m_chanArray[j-1]->getHorizontal()/SCREENNORM;
        
        m_chanArray[j]->tick(m_mixBufArray[j], bufferSize);

        for (int k = 0; k<bufferSize; k++){
            buffer[NCHAN * k] += (.2) * m_mixBufArray[j][k];
//            buffer[NCHAN * k] += (1 - offset) * (.2) * m_mixBufArray[j][k];
            for (int l = 1; l<NCHAN; l++)
                buffer[NCHAN*k+l] =(.2) * m_mixBufArray[j][k];
//                buffer[NCHAN*k+l] =(offset) * (.2) * m_mixBufArray[j][k];
        }
    }
    }
}

void Sinkapater::toggleAnimate(){
    // toggle animate mode when "T" is typed
    m_animate = !m_animate;
    
    for (int i = 0; i < m_numFaucets; i++){        
        m_chanArray[i]->setAnimate(m_animate);
    }  
    
    if (m_animate){
        // lighting stuff
        GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
        GLfloat mat_shininess[] = { 50.0 };
        GLfloat light_position[] = { 1.0, 1.0, 1.0, 0.0 };
        
        glClearColor (0.0, 0.0, 0.0, 0.0);
        glShadeModel (GL_SMOOTH);
        
        glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
        glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);
        glLightfv(GL_LIGHT0, GL_POSITION, light_position);
        
        glEnable(GL_LIGHTING);
        glEnable(GL_LIGHT0);
        glEnable(GL_DEPTH_TEST);
        
        // change perspective:
        ////////////////////
        // for perspective :
        glViewport( 0, 0, WIDTH, HEIGHT );
        // set the matrix mode to project
        glMatrixMode( GL_PROJECTION );
        // load the identity matrix
        glLoadIdentity( );
        // create the viewing frustum
        gluPerspective( 45.0, (GLfloat) WIDTH / (GLfloat) HEIGHT, 1.0, 100.0 );
        // set the matrix mode to modelview
        glMatrixMode( GL_MODELVIEW );
        // load the identity matrix
        glLoadIdentity( );
        // position the view point
        gluLookAt( 4.0f, 4.0f, 10.0f, 4.0f, 4.0f, 0.0f, 0.0f, 1.0f, 0.0f );
        
        printf("visualization mode!\n");
    }

    else {
        printf("edit mode!\n");
        
        glDisable(GL_LIGHTING);
        glDisable(GL_LIGHT0);
        glDisable(GL_DEPTH_TEST);
        
        // 
        glViewport(0, 0, (GLsizei) WIDTH, (GLsizei) HEIGHT);
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        glOrtho(0.0, SCREENNORM, 0.0, 8.0, -0.5, 2.5);
        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
    }
        
    
}

void Sinkapater::togglePlay(){
    // toggle animate mode when "T" is typed
    m_play = !m_play;
    
    for (int i = 0; i < m_numFaucets; i++){        
        m_chanArray[i]->setPlay(m_play);
    }  
    
    if (m_play)
        printf("Play!\n");
    else 
        printf("Pause!\n");
    
}